// File edited 02/20/2015 with four space tab spacing

/** \file
	\brief Main file - this is where it all starts, and ends
*/

#include "simpletools.h"
#include "config.h"
#include "dda_queue.h"
#include "dda.h"
#include "gcode_parse.h"
#include "debug.h"
#include "pinio.h"
#include "sergcode.h"
#include "Adafruit_ADS1015.h"
#include "clock_timer.h"
#include "step_timer.h"

/// initialise various I/O - set pins as input or output, turn off unused subsystems, etc
void io_init(void)
{
	// setup I/O pins

	// X Stepper
	SET_DIRA_OUTPUT(X_STEP_PIN);
	SET_OUTA(X_STEP_PIN, 0);

	SET_DIRA_OUTPUT(X_DIR_PIN);
	SET_OUTA(X_DIR_PIN,  0);	

	#ifdef X_MIN_PIN
		SET_DIRA_INPUT(X_MIN_PIN);
		SET_OUTA(X_MIN_PIN, 0);
	#endif

	#ifdef X_MAX_PIN
		SET_DIRA_INPUT(X_MAX_PIN);
		SET_OUTA(X_MAX_PIN, 0);
	#endif

	// Y Stepper
	SET_DIRA_OUTPUT(Y_STEP_PIN);
	SET_OUTA(Y_STEP_PIN, 0);

	SET_DIRA_OUTPUT(Y_DIR_PIN);
	SET_OUTA(Y_DIR_PIN, 0);

	#ifdef Y_MIN_PIN
		SET_DIRA_INPUT(Y_MIN_PIN);
		SET_OUTA(Y_MIN_PIN, 0);
	#endif

	#ifdef Y_MAX_PIN
		SET_DIRA_INPUT(Y_MAX_PIN);
		SET_OUTA(Y_MAX_PIN, 0);
	#endif

	// Z Stepper
	SET_DIRA_OUTPUT(Z_STEP_PIN);
	SET_OUTA(Z_STEP_PIN, 0);
	
	SET_DIRA_OUTPUT(Z_DIR_PIN);
	SET_OUTA(Z_DIR_PIN, 0);	

	#ifdef Z_MIN_PIN
		SET_DIRA_INPUT(Z_MIN_PIN);
		SET_OUTA(Z_MIN_PIN, 0);
	#endif

	#ifdef Z_MAX_PIN
		SET_DIRA_INPUT(Z_MAX_PIN);
		SET_OUTA(Z_MAX_PIN, 0);
	#endif

	// E Stepper
	SET_DIRA_OUTPUT(E_STEP_PIN);
	SET_OUTA(E_STEP_PIN, 0);

	SET_DIRA_OUTPUT(E_DIR_PIN);
	SET_OUTA(E_DIR_PIN, 0);

	// Common Stepper Enable
	#ifdef STEPPER_ENABLE_PIN

		SET_DIRA_OUTPUT(STEPPER_ENABLE_PIN);

		#ifdef STEPPER_INVERT_ENABLE
			SET_OUTA(STEPPER_ENABLE_PIN, 0);
		#else
			SET_OUTA(STEPPER_ENABLE_PIN, 1);
		#endif
			
	#endif

	// X Stepper Enable
	#ifdef X_ENABLE_PIN

		SET_DIRA_OUTPUT(X_ENABLE_PIN);

		#ifdef X_INVERT_ENABLE
			SET_OUTA(X_ENABLE_PIN, 0);
		#else
			SET_OUTA(X_ENABLE_PIN, 1);
		#endif
		
	#endif

	// Y Stepper Enable
	#ifdef Y_ENABLE_PIN

		SET_DIRA_OUTPUT(Y_ENABLE_PIN);

		#ifdef Y_INVERT_ENABLE
			SET_OUTA(Y_ENABLE_PIN, 0);
		#else
			SET_OUTA(Y_ENABLE_PIN, 1);
		#endif
		
	#endif

	// Z Stepper Enable
	#ifdef Z_ENABLE_PIN

		SET_DIRA_OUTPUT(Z_ENABLE_PIN);

		#ifdef Z_INVERT_ENABLE
			SET_OUTA(Z_ENABLE_PIN, 0);
		#else
			SET_OUTA(Z_ENABLE_PIN, 1);
		#endif
		
	#endif

	// E Stepper Enable
	#ifdef E_ENABLE_PIN

		SET_DIRA_OUTPUT(E_ENABLE_PIN);

		#ifdef E_INVERT_ENABLE
			SET_OUTA(E_ENABLE_PIN, 0);
		#else
			SET_OUTA(E_ENABLE_PIN, 1);
		#endif
		
	#endif

	#ifdef	STEPPER_ENABLE_PIN
		power_off();
	#endif
}

/// Startup code, run when we come out of reset
void init(void)
{
	#ifdef GCODE_SER
 
	// set up serial
	serial_init();

	#endif

	// set up G-code parsing
	gcode_init();

	// set up inputs and outputs
	io_init();

	// set up clock timer
	clock_timer_init();

	// set up step timer
	step_timer_init();

	// set up dda
	dda_init();

	// set up adc
	adc_init();

	// say hi to host
	//2/25/2015//Bruce//serial_writestr_P(PSTR("start\nok\n"));
}

/// Parse GCODE from an sd card file
void parse_sd(void)
{
	#if defined SD_DO && defined SD_CLK && defined SD_DI && defined SD_CS

	// Mount SD card
	sd_mount(SD_DO, SD_CLK, SD_DI, SD_CS);
    
	FILE * pFile;
    
	int c;

	pFile = fopen("debug.txt","r");

	if(pFile == NULL)
	{
		print("Error opening file");
	}
	else
	{
		do
		{
			// if queue is full, there is no point in reading characters, so wait
			if(queue_full() == 0)
			{
				c = getc(pFile);            
				gcode_parse_char(c);
			}

			clock();
		}
		while(c != EOF);
        
		fclose (pFile);
	}
    
	#else
    
	print("The proper inputs and outputs must be defined for SD card usage");
    
	#endif
}

/// Parse GCODE from a serial connection
void parse_serial(void)
{
	char c;
 
	while(1)
	{
		c = fdserial_rxChar(sergcode);
    
		if((c != -1) && (queue_full() == 0))
		{
			gcode_parse_char(c);
		}
  
		clock();
	} 
}

/// this is where it all starts, and ends
///
/// just run init(), then run an endless loop in either parse_sd() or parse_serial(),
/// where we pass characters from either a serial connection or an sd card file to the 
/// gcode_parse_char() function, meanwhile also checking the clocks

int main(void)
{
//	debug_flags |= DEBUG_PID;

	debug_flags |= DEBUG_DDA;
	debug_flags |= DEBUG_POSITION;//gcode_process-M114//

	debug_flags |= DEBUG_ECHO;//gcode_parse//
	debug_flags |= DEBUG_STEP;//setTimer

	init();
 
	#ifdef GCODE_SD

	parse_sd();

	#elif defined GCODE_SER

	parse_serial();

	#endif
 
	return 0;
}


/*
int main(void)
{
    adc_init();
    
    //3/11/2015//Bruce//double extruder_fahrenheit;    
    uint8_t extruder_channel = 0;
    //3/11/2015//Bruce//uint8_t bed_channel = 1;    
    
    while(1)
    {
        UpdateTemp(extruder_channel);
        //extruder_fahrenheit = (((extruder_celsius * 9) / 5) + 32);

        print("Current temp: %f C\r\n", extruder_celsius);
        
        pause(500);
    }


    return 0;
}
*/